#ifndef UTILS_H
#define UTILS_H


// A helper class to save and load settings from an INI file somewhat automatically...
// Use:
// TRADITIONAL USE:
// Use LoadSettings("ThisValue", 3.5f); and SaveSettings("ThisValue", 3.5f);
// But you have to make sure to do that for all the values, and make sure the keys are the same.
//
// MORE ADVANCED:
// Instead, pass a pointer to the value you want to load/restore if you can. The rest is then automatic.
// At the start (for example, in OnInitDialog or somewhere like that), make repeated
// calls to LoadSettings() for each value you'd like to store, giving a pointer to the value,
// the name of the key to use, and the default value.
// CSettings will remember all these guys, and when you exit (for example in OnDestroy()), you
// simply call SaveSettings(). 
// For HWND, the position and size of the window are stored/restored, and a message is sent to the window
// automatically.

class CSettings
{
public: 
	CSettings(char* AppName = NULL);
	~CSettings();

	// Declare and load settings for float, int and string values.
	void LoadSettings(float* Value, char* KeyName, float DefValue);
	void LoadSettings(int* Value, char* KeyName, int DefValue);
	void LoadSettings(char* Value, char* KeyName, char* DefValue);
	// This one is special! It loads the saved position and size of the window. 
	void LoadSettings(HWND HWnd, char* KeyName, bool NoResize = false);

	// This will save this specific setting. CSetting will search through its list to find the
	// setting that corresponds to the passed address.
	void SaveSettings(void* Value);
	// This will save all the settings previously declared.
	void SaveSettings();

	// These are more traditional. They return the value, rather than storing it in a pointer.
	// Also, they won't remember anything, so SaveSettings() will not do anything.
	float LoadSettings(char* KeyName, float DefValue);
	int  LoadSettings(char* KeyName, int DefValue);
	char* LoadSettings(char* KeyName, char* DefValue);
	void SaveSettings(char* KeyName, float Value);
	void SaveSettings(char* KeyName, int Value);
	void SaveSettings(char* KeyName, char* Value);

	// This will open the ini file with the default editor.
	void OpenIniFile();


private:

	// This enum defines the kind of data we're saving.
	typedef enum
	{
		m_Float,
		m_Int,
		m_String,
		m_Hwnd,
	} AType;

	// This structure contains everything we need for saving/restoring a setting...
	typedef struct
	{
		void*	m_Value;	// Will point to the application data to store/restore.
		char	m_Key[64];	// Copy of the key to use.
		AType	m_Type;		// Type of the data
	} ASetting;
	
	char	m_AppName[MAX_PATH];
	char	m_IniFilePath[MAX_PATH];

	int			m_NumSettings;		// How many settings we're saving..
	ASetting	m_Settings[1024];	// The settings...

	// A helper function.
	void	DoSave(int ThisSetting);

};

// Start an app, wait for result or not... If not waiting, returns the handle to the process.
HANDLE StartApp(const char* AppName, const char* Arguments = NULL, bool wait = 0);

// Concatenates str1 and str2 and returns result.
char* CatStr(const char* str1, const char* str2);

// Concatenates str1, str2 and str3 and returns result.
char* CatStr(const char* str1, const char* str2, const char* str3);

// Concatenates str1 and str2 and returns result. Makes sure there's a / in between!
char* CatPath(const char* str1, const char* str2);

// Concatenates str1, str2 and str3 and returns result. Makes sure there's a / in between!
char* CatPath(const char* str1, const char* str2, const char* str3);

// Find extension in srt1.
char* GetExt(const char* str1);

// Find filename in full path. Ex: for str1 = "C:/temp/foo.bit" returns foo.
char* GetFname(const char* str1);

// Find filename+ext in full path. Ex: for str1 = "C:/temp/foo.bit" returns foo.bit.
char* GetFnameExt(const char* str1);

// Find full directory path. Ex: for str1 = "C:/temp/foo.bit" returns C:/temp/.
char* GetDirPath(const char* str1);

// Change extension. Ex: for str1 = "C:/temp/foo.bit" and Ext = ".vob" returns C:/temp/foo.vob
char* ChangeExt(const char* str1, const char* Ext);

// Checks whether path exists (either file or directory).
bool PathExists(const char* str1);

// Checks whether path is a directory.
bool IsDir(const char* str1);

// Checks whether file is empty. Returns true if the file is empty, or does not exist.
bool IsEmpty(const char* str1);

// Get file size in KBytes. 0 if file does not exist.
unsigned long GetFileSizekB(const char* str1);

// Empty this directory (but don't remove it).
void EmptyDir(const char* str1);

// This returns the current directory.
char* GetCurrentDir();

// Shut down the computer!
void	ShutDownComputer();

// This returns the IFO name corresponding to the VTS name, e.g. VTS_03_0.IFO for VTS_03_1.VOB.
char* GetIFO(const char* VTSName);
char* GetBUP(const char* VTSName);

// This opens a binary file stored in the resource area of the app. It extracts it from the
// resource, puts it in the %TEMP% directory, adds ext for an extension and uses the default
// app to open it. RSCID must be the resource ID. The resource must be in a custom BINARY
// section of the resources. Make sure RSCID is the ressource that corresponds to the binary
// file.
void OpenRSCFile(UINT RSCID, char* ext);


#endif